package com.jsh.erp.controller.stocktaking;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.jsh.erp.base.AjaxResult;
import com.jsh.erp.base.BaseController;
import com.jsh.erp.base.TableDataInfo;
import com.jsh.erp.datasource.dto.TaskStocktakingDTO;
import com.jsh.erp.datasource.dto.TaskStocktakingItemDTO;
import com.jsh.erp.datasource.entities.TaskStocktaking;
import com.jsh.erp.datasource.entities.TaskStocktakingItem;
import com.jsh.erp.datasource.vo.SpinnerVO;
import com.jsh.erp.query.LambdaQueryWrapperX;
import com.jsh.erp.service.TaskStocktakingItemService;
import com.jsh.erp.service.TaskStocktakingService;
import com.jsh.erp.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@RequestMapping(value = "/stocktaking")
@Api(tags = {"盘点接口"})
public class StocktakingController extends BaseController {

    @Resource
    private TaskStocktakingService taskStocktakingService;

    @Resource
    private UserService userService;

    @Resource
    private TaskStocktakingItemService taskStocktakingItemService;

    @ApiOperation("盘点任务列表")
    @PostMapping("/list")
    public TableDataInfo list(){
        startPage();
        List<TaskStocktaking> list = taskStocktakingService.list();
        return getDataTable(list);
    }

    @ApiOperation("新增盘点任务")
    @PostMapping("/add")
    public AjaxResult add(@RequestBody TaskStocktakingDTO taskStocktakingDTO) {
        taskStocktakingService.add(taskStocktakingDTO);
        return AjaxResult.success();
    }

    @ApiOperation("负责人下拉列表")
    @GetMapping("/creatorSpinnerList")
    public AjaxResult creatorSpinnerList() {
        List<SpinnerVO> spinnerVOList = userService.creatorSpinnerList();
        return AjaxResult.success(spinnerVOList);
    }

    /**
     * 查询任务详情
     * @param id
     * @return
     */
    @GetMapping("/detail/{id}")
    public AjaxResult detail(@PathVariable("id") Long id) throws Exception{
        return AjaxResult.success(taskStocktakingService.detail(id));
    }

    /**
     * 任务详情-商品列表
     * @param taskStocktakingId 任务ID
     * @return
     */
    @GetMapping("/detailByItemList/{taskStocktakingId}")
    public AjaxResult detailByItemList(@PathVariable("taskStocktakingId") Long taskStocktakingId) {
        return AjaxResult.success(taskStocktakingService.listByTaskStocktakingId(taskStocktakingId));
    }

    /**
     * 任务详情-商品列表-编辑
     * @return
     */
    @PostMapping("/itemUpdate")
    public AjaxResult itemUpdate(@RequestBody TaskStocktakingItemDTO taskStocktakingItemDTO) {
        taskStocktakingItemService.update(new UpdateWrapper<TaskStocktakingItem>()
                .set("new_inventory", taskStocktakingItemDTO.getNewInventory())
                .set("new_position", taskStocktakingItemDTO.getNewPosition())
                .set("difference_count", taskStocktakingItemDTO.getDifferenceCount())
                .set("difference_reason", taskStocktakingItemDTO.getDifferenceReason())
                .eq("id", taskStocktakingItemDTO.getId()));
        return AjaxResult.success();
    }

}
